<?php

namespace WPEP\Admin\Container;

use WPEP\Controller as WPEP_Controller;

abstract class LicenseAndUpdates {

  public function __construct() {
    $this->api = new LicenseAndUpdates\API( $this );
  }

  abstract public function get_product_name();

  public function get_plugin_alias() {
    return wpep_label_to_alias( $this->get_product_name() );
  }

  public function has_background_license_verification() {
    return true;
  }

  abstract public function get_plugin_version();

  abstract public function get_product_id();

  abstract public function get_options_prefix();

  public function get_license() {
    return get_option( $this->get_option_license_key(), '' );
  }

  public function prepare_license_status($licenseStatus) {
    return is_string($licenseStatus) ? md5($licenseStatus . '-wpep') : $licenseStatus;
  }

  public function is_valid_license() {
    $status = get_option( $this->get_option_license_key_status() );

    return $status !== false && $status == $this->prepare_license_status('valid' );
  }

  public function get_option_license_key() {
    return $this->get_options_prefix() . 'key';
  }

  public function get_option_license_key_bundled_products() {
    return $this->get_options_prefix() . 'bundled_products';
  }

  public function get_option_license_key_status() {
    return $this->get_options_prefix() . 'status_information';
  }

  public function get_option_license_last_verification() {
    return $this->get_options_prefix() . 'last_verification';
  }

  /**
   * @var LicenseAndUpdates\Administration
   */
  public $administration;
  /**
   * @var LicenseAndUpdates\API
   */
  public $api;
  /**
   * @var LicenseAndUpdates\Updater
   */
  public $updater;

  final public function setup_license_and_updates( $plugin_base_file_path = false, $has_administration = true ) {
    if( !is_admin() )
      return;

    if( $has_administration )
      $this->administration = new LicenseAndUpdates\Administration( $this );

    if( $plugin_base_file_path !== false )
      $this->updater = new LicenseAndUpdates\Updater( $this, $plugin_base_file_path );

    if( !$this->has_background_license_verification() )
      return;

   if( intval( get_option( $this->get_option_license_last_verification(), 0 ) ) + 86400 < time() )
      add_action( 'admin_init', [ $this, '_administration_init_refresh_license_status' ] );
  }

  public function _administration_init_refresh_license_status() {
    $this->refresh_license_status();
    update_option( $this->get_option_license_last_verification(), time() );
  }

  final public function activate_license( $license ) {
    $this->set_license( $license );

    $license_data = $this->api->activate_license();

    if( isset( $license_data->license ) && $license_data->license == 'valid' ) {
      $this->set_license_status( 'valid' );
    } else {
      $this->set_license_status( 'invalid' );
    }

    $this->set_license_bundled_products(
      isset( $license_data->bundled_products ) ? $license_data->bundled_products : false
    );

    return $this->is_valid_license();
  }

  /**
   * @param $license
   * @return $this
   */
  public function set_license( $license ) {
    update_option( $this->get_option_license_key(), $license );

    return $this;
  }

  /**
   * @param string $status
   * @return $this
   */
  final public function set_license_status( $status = 'valid' ) {
    update_option( $this->get_option_license_key_status(), $this->prepare_license_status( $status ) );

    return $this;
  }

  final public function get_license_bundled_products() {
    return get_option( $this->get_option_license_key_bundled_products(), [] );
  }

  /**
   * @param bool|array $product_list
   * @return $this
   */
  final public function set_license_bundled_products( $product_list ) {
    if( $product_list == false )
      delete_option( $this->get_option_license_key_bundled_products() );
    else
      update_option( $this->get_option_license_key_bundled_products(), $product_list );

    return $this;
  }

  /**
   * @return $this
   */
  final public function refresh_license_status() {
    $license_data = $this->api->get_license_status();

    if( !isset( $license_data->license ) )
      return $this;

    if( $license_data->license == 'inactive' ) {
      $this->activate_license( $this->get_license() );
    } else if( $license_data->license == 'valid' ) {
      $this->set_license_status( 'valid' );
    } else {
      $this->set_license_status( 'invalid' );
    }

    $this->set_license_bundled_products(
      isset( $license_data->bundled_products ) ? $license_data->bundled_products : false
    );

    return $this;

  }

}