<?php
namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;

class AddOnsSection {

  private $_transient_key = 'wpep_official_addon_list';

  public function setup() {
    add_action( 'admin_notices', [ $this, 'admin_notices' ], 15 );
    add_action( 'admin_menu',  [ $this, 'admin_menu' ], 15 );
  }

  public function admin_notices() {
    if( empty( WPEP_Controller::instance()->addOns->addOns_missing_dependencies )
        || !is_array( WPEP_Controller::instance()->addOns->addOns_missing_dependencies ) )
      return;

    $admin_notice_list = $this->_get_admin_notice_list();

    if( $admin_notice_list == '' )
      return;

    echo '<div class="error">';
    echo    '<h2>' . __( "WPEP AddOn Integrations Notice.", "wpep" ) . '</h2>';
    echo    '<p>' . __( "The following Integrations have not been loaded due to missing their integrated plugin, make sure it's active.", "wpep" ) . '</p>';
    echo    '<ul>';
    echo      $admin_notice_list;
    echo    '</ul>';
    echo '</div>';
  }

  private function _get_admin_notice_list() {
    $resp = '';

    foreach( WPEP_Controller::instance()->addOns->addOns_missing_dependencies as $addOnAlias ) {
      $addOn = WPEP_Controller::instance()->addOns->get_add_on( $addOnAlias );

      $requirements_response = $addOn->has_requirements_met();

      if( !is_array( $requirements_response ) )
        continue;

      $requirements_list = '';

      foreach( $requirements_response as $plugin_name => $plugin_url )
        $requirements_list .= '<a href="' . $plugin_url . '" target="_blank">' . $plugin_name . '</a>';

      $resp .= '<li>' . sprintf(
                  __( 'WPEP %s requires : %s ', 'wpep' ),
                  $addOn->get_name(),
                  $requirements_list
                ) . '</li>';
    }

    return $resp;
  }

  public function admin_menu() {
    add_submenu_page(
        'edit.php?post_type=' . WPEP_POST_TYPE_COURSE,
        'Add-ons',
        'Add-ons',
      WPEP_CAP_MANAGE_SETTINGS,
        'wpep-addons',
        [ $this, 'administration_page' ]
    );
  }

  public function administration_page() {
    printf( '<h2>%s</h2>', __('WPEP Add-ons', "wpep" ) );
    echo '<hr/>';

    echo '<div class="wrap">';
    foreach ($this->get_available_addon_list() as $product) {
      if ( null === $product )
        continue;

      $name  = trim(esc_html( str_ireplace( 'WpEp', '', $product->title ) ));
      $alias = str_ireplace( 'wpep-', '', $product->slug );

      $addon = WPEP_Controller::instance()->addOns->get_add_on_by_store_id( $product->id );

      echo '<div class="wpep-addon-card pwep_addon_prod">';
      echo  '<img src="' . esc_url($product->thumbnail) . '" />';
      echo  '<h3>' . $name . '</h3>';

      echo $this->get_addon_status_text( $addon );

      echo '<p class="wpep-plugin-management-operations">';

      echo $this->get_addon_status_operations( $alias, $addon, $name, $product->id );

      echo    ' <a href="' . esc_url($product->link) . '" target="_blank" class="button-secondary" >' . __('Details', "wpep") . '</a>';
      echo  '</p>';

      echo  '<p class="description">' . ( (!empty($product->excerpt)) ? $product->excerpt : __( "We don't have a short description just yet, please check the website for more information", "wpep" ) ) . '</p>';
      echo '</div>';
    }

    echo '</div>';
  }

  public function get_addon_image_by_controller_information( $information ) {
    if( !isset( $information[ 'store_id' ] ) )
      return '';

    if( !isset( $information[ 'official' ] ) || $information[ 'official' ] == false )
      return '';

    $addon_list = $this->get_available_addon_list();

    if( !isset( $addon_list[ $information['store_id'] ] ) )
      return '';

    return isset( $addon_list[ $information['store_id'] ]->thumbnail ) ? $addon_list[ $information['store_id'] ]->thumbnail : '';
  }

  public function get_available_addon_list() {
    $data = get_transient( $this->_transient_key );

    if (false === $data || empty($data) ) {
      $response         = wp_remote_post( WPEP_STORE_PRODUCTS_LINK, [ 'timeout' => 15, 'sslverify' => false ] );
      $response_entries = json_decode(wp_remote_retrieve_body($response));

      if( !isset( $response_entries->products ) )
        return [];

      foreach( $response_entries->products as $response_entry ) {
        if( !is_array( $response_entry->info->category  ) )
          continue;

        foreach( $response_entry->info->category as $category_information ) {
          if( $category_information->slug != 'addons' )
            continue;

          $data[] = $response_entry->info;
          continue 2;
        }

      }

      $data_ordered = wpep_post_list_ordering( $data, 'ASC', 'title', false );
      $data = [];

      foreach( $data_ordered as $value )
        $data[ $value->id ] = $value;

      set_transient( $this->_transient_key, $data, HOUR_IN_SECONDS * 24 );
    }

    return $data;
  }

  public function get_addon_status_text( $addon = false ) {
    if( $addon != false )
      return  '<p class="status status-active">' . __( 'Active', "wpep") . '</p>';
    else
      return  '<p class="status">' . __( 'Inactive', "wpep") . '</p>';
  }

  public function get_addon_status_operations( $alias, $addon = false, $name = false, $product_id = false ) {
    if( $addon != false ) {

      $settings_page = WPEP_Controller::instance()->addOns->get_add_on_settings( $addon->get_alias() );

      if( $settings_page != false )
        return '<a href="' . $settings_page->get_page_link() . '" class="button-primary" >' . __( 'Settings', "wpep") . '</a>';

      return '';
    }

    if( !current_user_can( 'activate_plugins') )
      return '';

    $plugin_activation = $this->_get_addon_status_operations_activation( $alias, $name );

    if( $plugin_activation != '' )
      return $plugin_activation;

    if( !in_array( $product_id, wpep_controller()->adminController->license->get_license_bundled_products() ) )
      return '';

    if( !current_user_can( 'install_plugins') )
      return '';

    return '<a data-wpep-plugin-action-install="' . $product_id . '" 
               data-wpep-alias="' . $alias . '"
               data-wpep-nonce="' . wp_create_nonce( 'wpep-install-plugin_' . $product_id ) . '"
               class="button-primary" >' . __( 'Install', "wpep") . '</a>';
  }

  private function _get_addon_status_operations_activation( $alias, $name = false, $depth = 0 ) {
    $plugin_file_identifier = 'wpep-' . $alias . '/' . 'wpep-' . $alias . '.php';

    if( file_exists( WP_PLUGIN_DIR . '/' . $plugin_file_identifier ) ) {
      return  '<a data-wpep-plugin-action-activate="' . $plugin_file_identifier . '"
                  data-wpep-alias="' . $alias . '"
                  data-wpep-nonce="' . wp_create_nonce( 'activate-plugin_' . $plugin_file_identifier ) . '" class="button-primary" >' . __('Activate', "wpep") . '</a>';
    }

    if( $depth == 0 && strpos( $alias, '-' ) !== false ) {
      $alias = substr_replace( $alias, '', strpos( $alias, '-' ), 1 );

      return $this->_get_addon_status_operations_activation( $alias, $name, $depth + 1 );
    }

    if( $name == false )
      return '';

    $alias = sanitize_title( $name );
    $alias = str_replace( [ 'drag-drop-' ], '', $alias );

    return $this->_get_addon_status_operations_activation( $alias, false, $depth + 1 );
  }
}