<?php

namespace WPEP\Coordinator\Content;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;
use WPEP\Coordinator\Content\Single\Skeleton as WPEP_SC_Skeleton;


class Single {

  /**
   * @var WPEP_SC_Skeleton
   */
  protected $currentDelivery = false;
  protected $currentDeliveryAlias = false;

  public $current_course_id = false;
  public $current_course_sidebar_option = false;
  
  public $current_prerequisite_mode    = null;
  public $current_prerequisites        = [];
  public $current_prerequisites_locked = false;

  public function __construct() {
    if( is_admin() && !( defined( 'DOING_AJAX' ) && DOING_AJAX ) )
      return;

    add_filter( 'init', [ $this, 'init' ], 15 );

    if( wpep_get_setting_flag( 'course-use-the-content-filter' ) )
      add_filter( 'the_content', [ $this, 'the_content_filter' ], 1 );

    add_action( 'wpep_single_content_sidebar', [ $this, 'action_wpep_single_sidebar' ], 10 );
  }

  public function get_current_course_id() {
    return $this->current_course_id;
  }
  
  public function init( $current_course_id = false ) {
    if( $current_course_id == false )
      $current_course_id = wpep_get_current_post_id();

    if( $this->current_course_id == $current_course_id
        && wpep_course_type_from_id( $current_course_id ) == $this->currentDeliveryAlias )
      return;

    $this->current_course_id = $current_course_id;

    if( get_post_type( $this->current_course_id ) != WPEP_POST_TYPE_COURSE )
      return;

    if( is_user_logged_in() && isset( $_GET['wpep-login'] ) ) {
      $redirect = get_permalink( $current_course_id );

      if( wpep_has_embed_support() && isset( $_GET['wpep-embed'] ) )
        $redirect = add_query_arg( 'wpep-embed', 1, $redirect );

      wp_redirect( $redirect );
      exit;
    }

    if( wpep_post_has_member_magnet_option( $this->current_course_id, get_current_user_id() ) ) {
      if( !in_array( wpep_course_type_from_id( $this->current_course_id ), wpep_course_type_member_magnet_integrated() )  ) {
        if( !isset( $_GET['wpep-login'] ) ) {
          wp_redirect( wpep_post_member_magnet_link( $this->current_course_id ) );
          exit;
        }
      }

      wpep_current_request_setup_member_magnet();
    }

    $course_theme = get_post_meta( $this->current_course_id, 'course_theme', true );

    if( $course_theme !== '' && !WPEP_Controller::instance()->templateStyle->custom_style_delivered )
      WPEP_Controller::instance()->templateStyle->theme = $course_theme;

    $this->refresh_current_delivery();

    remove_filter ('the_content', 'wpautop');
  }

  public function refresh_current_delivery() {
    if( $this->currentDelivery !== false ) {
      remove_filter( 'wpep_single_content_type_alias', [ $this->currentDelivery, 'get_alias' ] );

      if( method_exists( $this->currentDelivery, 'remove_init' ) )
        $this->currentDelivery->remove_init();

      unset( $this->currentDelivery );
    }

    $this->currentDelivery = $this->get_course_type_coordinator( $this->current_course_id );

    if( $this->currentDelivery == false )
      wp_die( __( "There is something wrong with this course, please check contact the owner and notify him." ), "wpep" );

    $this->_setup_prerequisites();
    $this->before_currently_deliver_init();

    $this->currentDelivery->init( $this->current_course_id );

    add_filter( 'wpep_single_content_type_alias', [ $this->currentDelivery, 'get_alias' ] );
  }

  public function the_content_filter( $content ) {
    if( post_password_required() )
      return $content;

    if( get_post_type() !== WPEP_POST_TYPE_COURSE )
      return $content;

    if( $this->currentDelivery == false )
      return $content;

    if( apply_filters( 'wpep_single_content_course_ignore_delivery', ( wpep_get_setting( 'course-layout-content-editor' ) && strpos( $content, '[wpep_the_course]' ) !== false ) ) )
      return $content;

    if( $this->current_prerequisites_locked )
      return $this->_get_prerequisites_content();

    return $this->_get_the_content();
  }

  private function _get_the_content() {
    $content_after = "";

    if( wpep_has_engagement_embed_support()
        && (
          !isset( $_GET['wpep-embed'] )
            || wpep_get_setting_flag( 'user-content-embed-box-inside-embeds' )
        ) ) {
      $embed_post_id = wpep_get_current_post_id();

      if( $embed_post_id === 0 )
        $embed_post_id = get_the_ID();

      if( $embed_post_id != 0 ) {
        $content_after .= WPEP_Controller::instance()->template->get_template('partial/user-engagement-embed.php', [
          'embed_post_id' => $embed_post_id
        ] );
      }
    }

    if( $this->currentDelivery->has_next_suggestions_after_completion == false )
      $content_after .= $this->_get_recommended_follow_up_content();

    if( WPEP_Controller::instance()->userActivity->has_completed_course( $this->get_current_course_id(), get_current_user_id() ) ) {
      $content_after .= wpep_prepare_tracking_code(
        get_post_meta( $this->get_current_course_id(), 'wpep_tracking_code_completed', true ),
        'course-completed'
      );
    }

    return apply_filters( 'wpep_single_content_course_before_html', "", $this->current_course_id ) .
           $this->currentDelivery->content() .
           apply_filters( 'wpep_single_content_course_after_html', is_string( $content_after ) ? $content_after : "", $this->current_course_id );
  }

  /**
   * @return WPEP_SC_Skeleton
   */
  public function get_course_type_coordinator( $current_course_id ) {
    $coordinator_object = false;

    $current_course_type = wpep_course_type_from_id( $current_course_id );

    if( $current_course_type == WPEP_COURSE_TYPE_EXECUTION_PLAN )
      $coordinator_object = new Single\ExecutionPlan( $current_course_id );
    else if( $current_course_type == WPEP_COURSE_TYPE_EXECUTION_PLAN_UNFOLDED )
      $coordinator_object = new Single\ExecutionPlanUnfolded( $current_course_id );
    else if( $current_course_type == WPEP_COURSE_TYPE_STANDARD )
      $coordinator_object = new Single\Standard( $current_course_id );
    else if( $current_course_type == WPEP_COURSE_TYPE_VIDEO )
      $coordinator_object = new Single\Video( $current_course_id );
    else if( $current_course_type == WPEP_COURSE_TYPE_ONE_OFF )
      $coordinator_object = new Single\Basic( $current_course_id );
    else if( $current_course_type == WPEP_COURSE_TYPE_LEARN_MAP )
      $coordinator_object = new Single\LearnMap( $current_course_id );

    $this->currentDeliveryAlias = $current_course_type;

    return apply_filters( 'wpep_single_content_coordinator_object', $coordinator_object, $current_course_id );
  }

  /**
   * @return WPEP_SC_Skeleton
   */
  public function getCurrentDeliveryMethod() {
    return $this->currentDelivery;
  }

  public function getCurrentDeliveryMethodAlias() {
    return $this->currentDeliveryAlias;
  }

  public function before_currently_deliver_init() {
    $this->current_course_sidebar_option = ( $this->current_course_sidebar_option == false ? get_post_meta( $this->current_course_id, 'wpep_sidebar_position', true ) : $this->current_course_sidebar_option );

    if( $this->current_course_sidebar_option != '' && $this->current_course_sidebar_option != 'default' ) {
      add_filter( 'wpep_single_content_sidebar_active', [ $this, 'single_content_sidebar_active_filter' ] );
      add_filter( 'wpep_single_content_type_attr_classes', [ $this, 'single_content_type_classes_filter' ] );
    }
  }

  public function action_wpep_single_sidebar() {
    if( !wpep_content_primary_sidebar_active() )
      return;

    echo '<div class="course-sidebar-content" role="complementary">';
    echo    '<div class="wpep-widget-area">';

    do_action( 'wpep_single_sidebar_before' );

    if ( is_active_sidebar('wpep-sidebar') )
      dynamic_sidebar('wpep-sidebar');

    do_action( 'wpep_single_sidebar_after' );

    echo    '</div>';
    echo '</div>';

    return;
  }

  public function single_content_sidebar_active_filter( $status ) {
    if( $this->current_course_sidebar_option == 'disabled' )
      return false;

    if( $this->current_course_sidebar_option != '' && is_active_sidebar('wpep-sidebar') )
      return true;

    return $status;
  }

  public function single_content_type_classes_filter( $classes = [] ) {
    $classes['sidebar-position'] = 'sidebar-position-' . $this->current_course_sidebar_option;

    return $classes;
  }

  private function _get_prerequisites_content() {
    WPEP_Controller::instance()->notifications->add_notification(
      sprintf( wpep_get_setting( 'prerequisite-required-notification' ),
        '<a href="' . get_permalink( $this->current_course_id ). '"><strong>' . get_the_title( $this->current_course_id ) . '</strong></a>'
      ),
      'warning'
    );

    return wpep_content_primary_course_list_by_ids( $this->current_prerequisites );
  }

  private function _setup_prerequisites() {
    $this->current_prerequisites = apply_filters( "wpep_single_course_prerequisites_list", get_post_meta( $this->current_course_id, 'wpep_prerequisite' ) );

    if( empty( $this->current_prerequisites ) )
      return;

    if( current_user_can( WPEP_CAP_EDIT_COURSE ) ) {
      $course_string = '';

      foreach( $this->current_prerequisites as $prerequisite ) {
        $course_string .= ( $course_string == '' ? '' : ', ' );
        $course_string .= '<a href="' . get_the_permalink( $prerequisite ) . '">';
        $course_string .=   get_the_title( $prerequisite );
        $course_string .=   ' ( ' . wpep_get_percentage( WPEP_Controller::instance()->userActivity->course_progress_percentage( $prerequisite, get_current_user_id() ) ) . ' )';
        $course_string .= '</a>';
      }

      WPEP_Controller::instance()->notifications->add_notification(
        sprintf( __( 'Admin Only Notification. This course has the following prerequisites : %s'), $course_string ),
        'info'
      );

      return;
    }

    $this->current_prerequisite_mode = wpep_get_prerequisite_mode( get_post_meta( $this->current_course_id, WPEP_PREREQUISITE_MODE_POST_META, true ) );

    $completed_prerequisites = [];

    foreach( $this->current_prerequisites as $prerequisite ) {
      if( !WPEP_Controller::instance()->userActivity->has_completed_course( $prerequisite, get_current_user_id(), true ) )
        continue;

      $completed_prerequisites[] = $prerequisite;
    }

    if( $this->current_prerequisites_locked )
      $this->current_course_sidebar_option = 'disabled';

    $requirement = ( $this->current_prerequisite_mode[WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT] === 'all' ? count( $this->current_prerequisites ) : $this->current_prerequisite_mode[WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT] );

    $this->current_prerequisites_locked = ( $requirement != 0 && $requirement > count( $completed_prerequisites )  );

    if( $this->current_prerequisites_locked )
      return;

    if( !$this->current_prerequisite_mode[ WPEP_PREREQUISITE_FIELD_SHOW_NOTIFICATION ] )
      return;

    $course_string = '';

    foreach( $this->current_prerequisites as $prerequisite ) {
      if( in_array( $prerequisite, $completed_prerequisites ) )
        continue;

      $course_string .= ( $course_string == '' ? '' : ', ' );
      $course_string .= '<a href="' . get_the_permalink( $prerequisite ) . '">';
      $course_string .=   get_the_title( $prerequisite );
      $course_string .=   ' ( ' . wpep_get_percentage( WPEP_Controller::instance()->userActivity->course_progress_percentage( $prerequisite, get_current_user_id() ) ) . ' )';
      $course_string .= '</a>';
    }

    if( $course_string != '' )
      WPEP_Controller::instance()->notifications->add_notification(
        sprintf( wpep_get_setting_translated('prerequisite-suggested-notification'), $course_string ),
        'info'
        );
  }

  private function _get_recommended_follow_up_content() {
    if( !apply_filters( "wpep_single_course_has_next_suggestions", $this->currentDelivery->has_next_suggestions ) )
      return '';

    $recommended_courses = get_posts([
      'post_type'   => WPEP_POST_TYPE_COURSE,
      'numberposts' => -1,
      'meta_key'    => 'wpep_prerequisite',
      'meta_value'  => $this->current_course_id,
      'fields'      => 'ids'
    ]);

    if( empty( $recommended_courses ) )
      return '';

    $content  = '';
    $content .= '<div class="wpep-follow-up-content wpep-follow-up-content-on-course-' . $this->currentDelivery->get_alias() . '">';
    $content .=   '<h2>' . wpep_get_setting_translated('prerequisite-next-courses-title' ). '</h2>';
    $content .=   wpep_content_primary_course_list_by_ids( $recommended_courses, [
      'lock_by_prerequisites' => 0
    ] );
    $content .= '</div>';

    return $content;
  }

}