<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;

class Search {

  public $post_types = [
    WPEP_POST_TYPE_COURSE,
    WPEP_POST_TYPE_VIDEO,
    WPEP_POST_TYPE_E_BOOK,
    WPEP_POST_TYPE_OFFER
  ];

  /**
   * @param $keyword
   * @param array $post_types
   * @param int $result_entries_count
   * @param array $search_rules
   * @return array
   */
  public function get_as_array( $keyword, $post_types = [], $result_entries_count = 10, $search_rules = [] ) {
    $post_list = $this->get_as_posts( $keyword, $post_types, $result_entries_count, $search_rules );
    $response  = [];

    foreach( $post_list as $post ) {
      $response[ $post->ID ] = [
        'ID'        => $post->ID,
        'post_type' => $post->post_type,
        'title'     => $post->post_title,
        'thumbnail' => WPEP_PLACEHOLDER_IMAGE,
        'permalink' => false,
        'thumbnail_ribbon_text' => '',
      ];

      if( $post->post_type === WPEP_POST_TYPE_COURSE ) {

        $response[ $post->ID ]['permalink'] = apply_filters( 'wpep_course_link', get_permalink( $post ) , $post->ID );
        $response[ $post->ID ]['thumbnail'] = wpep_course_thumbnail_url( $post->ID );

        if( wpep_get_setting_flag( 'search-image-ribbon' ) )
          $response[ $post->ID ]['thumbnail_ribbon_text'] = get_post_meta( $post->ID, 'wpep_course_ribbon_text', true );

        if( $response[ $post->ID ]['thumbnail'] === '' )
          $response[ $post->ID ]['thumbnail'] = WPEP_PLACEHOLDER_IMAGE;

      } else if( $post->post_type === WPEP_POST_TYPE_E_BOOK ) {
        $thumbnail_id = get_post_thumbnail_id( $post->ID );

        if( $thumbnail_id != '' )
          $response[ $post->ID ]['thumbnail'] = get_post( $thumbnail_id, 'full' )->guid;

        $response[ $post->ID ]['permalink'] = apply_filters(
          'wpep_ebook_link',
          ( wpep_get_setting_flag( 'ebook-redirect-display' ) ? get_permalink( $post ) : get_post_meta( $post->ID, '_ebook_link', true ) ),
          $post->ID
        );

      } else if( $post->post_type === WPEP_POST_TYPE_OFFER ) {

        $response[ $post->ID ]['permalink'] = apply_filters( 'wpep_offer_link', get_permalink( $post ) , $post->ID );

        $thumbnail_id = get_post_thumbnail_id( $post->ID );

        if( $thumbnail_id != '' )
          $response[ $post->ID ]['thumbnail'] = get_post( $thumbnail_id, 'full' )->guid;

      } else if( $post->post_type === WPEP_POST_TYPE_VIDEO ) {

        $response[ $post->ID ]['permalink'] = apply_filters( 'wpep_video_link', get_permalink( $post ) , $post->ID );

      }
    }

    return $response;
  }

  /**
   * @param $keyword
   * @param array $post_types
   * @param int $result_entries_count
   * @param array $search_rules
   * @return array
   */
  public function get_as_posts( $keyword, $post_types = [], $result_entries_count = 10, $search_rules = [] ) {
    if( is_string( $post_types ) && $post_types != '' )
      $post_types = explode( ",", $post_types );

    if( !is_array( $post_types ) || empty( $post_types ) ) {
      $post_types = $this->post_types;
    } else {
      foreach( $post_types as $k => $post_type )
        if( !in_array( $post_type, $this->post_types ) )
          unset( $post_types[ $k ] );
    }

    $keyword = sanitize_text_field( $keyword );

    $search_pattern = "%" . $keyword . "%";

    global $wpdb;

    $sql = 'SELECT post_id, 
                   COUNT( post_id ) as matches 
              FROM (
                SELECT ID as post_id FROM ' . $wpdb->posts . '
                         WHERE post_title LIKE "' . $search_pattern . '"
                           AND post_type IN ("' . implode( '", "', $post_types ) . '")
                           AND post_status = "publish"
             ';

    if( in_array( WPEP_POST_TYPE_COURSE, $post_types ) && wpep_get_setting_flag( 'search-course-content' ) ) {
      $sql .= '   UNION ALL
                  SELECT post_id FROM ' . $wpdb->prefix . 'wpep_section
                                WHERE title   LIKE "' . $search_pattern . '"
                                   OR content LIKE "' . $search_pattern . '"
                  UNION ALL
                  SELECT post_id FROM ' . $wpdb->prefix . 'wpep_section_lesson
                                WHERE title   LIKE "' . $search_pattern . '"
                                   OR content LIKE "' . $search_pattern . '"';
    }

    if( in_array( WPEP_POST_TYPE_COURSE, $post_types ) && wpep_get_setting_flag( 'search-course-description' ) ) {
      $sql .= '   UNION ALL
                  SELECT post_id FROM ' . $wpdb->postmeta . '
                                WHERE ( meta_key = "wpep_course_description" AND meta_value LIKE "' . $search_pattern . '" )
                                  OR ( meta_key = "_wpep_course_video_jump_links" AND meta_value LIKE "' . $search_pattern . '" )';
    }

    $sql .= ' ) as results
            GROUP BY post_id
            ORDER BY matches DESC';

    $search_results = $wpdb->get_results( $sql );

    if( empty( $search_results ) )
      return [];

    $search_results = wpep_utility_map_to_array_assoc( $search_results, 'post_id', 'matches' );

    $args = [
      'post_type'   => 'any',
      'post_status' => 'publish',
      'include'     => array_keys( $search_results ),
      'meta_query'  => [
        'wpep_search' => [
          'relation' => 'OR',
          [
            'key'     => 'wpep_show_on_index',
            'value'   => '1',
            'compare' => '=',
          ],
          [
            'key'     => 'wpep_show_on_index',
            'compare' => 'NOT EXISTS'
          ]
        ]
      ],
      'paged'          => false,
      'posts_per_page' => -1
    ];

    if( isset( $search_rules['include_categories'] ) && $search_rules['include_categories'] != '' ) {
      $include_categories = $search_rules['include_categories'];

      if( !is_array( $include_categories ) )
        $include_categories = explode( ",", $include_categories );

      $include_categories = array_map( 'intval', $include_categories );

      $args['tax_query'] = [
        'wpep_search_include_categories'  => [
          'taxonomy'         => WPEP_TAXONOMY_CATEGORY,
          'field'            => 'id',
          'terms'            => $include_categories,
          'operator'         => 'IN'
        ]
      ];
    } else if( isset( $search_rules['exclude_categories'] ) && $search_rules['exclude_categories'] != '' ) {
      $exclude_categories = $search_rules['exclude_categories'];

      if( !is_array( $exclude_categories ) )
        $exclude_categories = explode( ",", $exclude_categories );

      $exclude_categories = array_map( 'intval', $exclude_categories );

      $args['tax_query'] = [
        'wpep_search_exclude_categories'  => [
          'taxonomy'         => WPEP_TAXONOMY_CATEGORY,
          'field'            => 'id',
          'terms'            => $exclude_categories,
          'operator'         => 'NOT IN'
        ]
      ];
    }

    $response = get_posts( $args );

    foreach( $response as $key => $post )
      $response[$key]->matches = $search_results[ $post->ID ];

    $response = wpep_post_list_ordering( $response, 'DESC', 'matches', false );

    return array_slice( $response, 0, $result_entries_count );
  }

}