<?php

abstract class WPEP_Post_Comments_Integration {

  public $comments_hierarchical = 'threaded';

  public function __construct() {
    add_filter( "wpep_post_comments_register_instance", [ $this, '_register_instance'] );
  }

  public function _register_instance( $instances ) {
    $instances[ $this->get_namespace() ] = $this;

    return $instances;
  }

  public function show_welcome_message() {}

  abstract public function get_namespace();
  abstract public function current_user_can_view( $post_id );
  abstract public function current_user_can_post( $post_id );

  /**
   * Implement this method if required.
   * @param $comment
   * @param $post_id
   * @param $user_id
   * @return bool
   */
  public function _after_comment_inserted( $comment, $post_id, $user_id ) {
    return false;
  }

  public function has_comment_type( $type ) {
    if( $type === '' )
      return true;

    $comment_types = $this->get_comment_types();

    return isset( $comment_types[ $type ] );
  }

  public function get_comment_types() {
    return [];
  }

  /**
   * @param bool $post_id
   * @param bool $list_only
   * @param null $hierarchical
   * @return string
   */
  public function get_comments_template( $post_id = false, $list_only = false, $hierarchical = null ) {
    if( $post_id !== false && get_the_ID() != $post_id ) {
      global $post;

      $previous_post = $post;
      $post = get_post( $post_id );
    }

    ob_start();

    $this->load_comments_template( $list_only, $hierarchical );

    $ret = ob_get_contents();

    ob_end_clean();

    if( isset( $previous_post ) ) {
      $post = $previous_post;
      setup_postdata( $post );
    }

    return $ret;
  }

  /**
   * @param bool $list_only
   * @param null $hierarchical
   */
  public function load_comments_template( $list_only = false, $hierarchical = null ) {
    $comments_template = ( $list_only ? 'comments_template_list_only_filter' : 'comments_template_filter' );

    if( $hierarchical !== null ) {
      $previous_hierarchical = $hierarchical;

      $this->comments_hierarchical = $hierarchical;
    }

    // We want the comments_template to work even in AJAX Requests
    global $withcomments;

    $previous_withcomments = $withcomments;
    $withcomments = true;

    add_action( 'wpep_post_comments_redaction_top', [ $this, 'show_welcome_message' ], 20 );
    add_filter( 'comments_template', [ $this, $comments_template ], 100 );
    add_filter( 'wpep_post_comments_namespace_alias', [ $this, 'get_namespace'], 100 );
    add_filter( 'wpep_post_comments_current_page', [ $this, 'get_current_page' ], 100 );
    add_filter( 'comments_template_query_args', [ $this, 'template_query_args'], 100 );
    add_filter( 'pre_option_page_comments', '__return_true', 100 );

    comments_template();

    remove_action( 'wpep_post_comments_redaction_top', [ $this, 'show_welcome_message' ], 20 );
    remove_filter( 'comments_template', [ $this, $comments_template ], 100 );
    remove_filter( 'wpep_post_comments_namespace_alias', [ $this, 'get_namespace'], 100 );
    remove_filter( 'wpep_post_comments_current_page', [ $this, 'get_current_page' ], 100 );
    remove_filter( 'comments_template_query_args', [ $this, 'template_query_args'], 100 );
    remove_filter( 'pre_option_page_comments', '__return_true', 100 );

    $withcomments = $previous_withcomments;

    update_user_meta( get_current_user_id(), 'wpep_post_comments_interaction_'. get_the_ID(), time() );

    if( isset( $previous_hierarchical ) )
      $this->comments_hierarchical = $previous_hierarchical;
  }

  public function comments_template_filter() {
    return WPEP\Controller::instance()->template->locate_template( 'comments/template.php' );
  }

  public function comments_template_list_only_filter() {
    return WPEP\Controller::instance()->template->locate_template( 'comments/list.php' );
  }

  public function template_query_args( $comment_args ) {
    $per_page = wpep_get_setting( 'comments-per-page' );

    $comment_args[ 'order' ]   = 'DESC';
    $comment_args[ 'orderby']  = 'comment_date';
    $comment_args[ 'number' ]  = $per_page;
    $comment_args[ 'offset' ]  = ( $this->get_current_page() - 1 ) * $per_page;
    $comment_args[ 'search' ]  = $this->get_search_term();
    $comment_args[ 'hierarchical' ] = $this->comments_hierarchical;
    $comment_args[ 'type' ]         = $this->get_comment_type();
    $comment_args[ 'parent__in']    = $this->get_comment_parent__in();
    $comment_args[ 'date_query']    = $this->get_comment_date_query();

    if( isset( $_POST[ 'wpep_post_comments_sync' ] ) && $_POST[ 'wpep_post_comments_sync' ] )
      $comment_args[ 'hierarchical' ] = false;

    return $comment_args;
  }

  public function get_current_page() {
    if( isset( $_POST[ 'wpep_post_comments_page' ] ) )
      return intval( $_POST[ 'wpep_post_comments_page' ] );

    if( isset( $_GET[ 'wpep-post-comments-page' ] ) )
      return intval( $_GET[ 'wpep-post-comments-page' ] );

    return 1;
  }

  public function get_search_term() {
    if( isset( $_POST[ 'wpep_post_comments_search'] ) )
      return sanitize_text_field( $_POST[ 'wpep_post_comments_search'] );

    return '';
  }

  public function get_comment_type() {
    $comment_type = '';

    if( isset( $_POST[ 'wpep_post_comments_type'] ) )
      $comment_type = sanitize_text_field( $_POST[ 'wpep_post_comments_type'] );
    else if( isset( $_GET[ 'wpep-post-comments-type'] ) )
      $comment_type = sanitize_text_field( $_GET[ 'wpep-post-comments-type'] );

    return $this->has_comment_type( $comment_type ) ? $comment_type : '';
  }

  public function get_comment_parent__in() {
    $parent_ids = [];

    if( isset( $_POST[ 'wpep_post_comments_parent_ids'] ) )
      $parent_ids = $_POST[ 'wpep_post_comments_parent_ids'];
    else if( isset( $_GET[ 'wpep-post-comments-parent-ids'] ) )
      $parent_ids = $_POST[ 'wpep-post-comments-parent-ids'];

    if( empty( $parent_ids ) )
      return '';

    $parent_ids = is_string( $parent_ids ) ? explode( ",", $parent_ids ) : $parent_ids;

    foreach( $parent_ids as $parent_id_key => $parent_id )
      $parent_ids[ $parent_id_key ] = intval( $parent_id );

    return empty( $parent_ids ) ? '' : $parent_ids;
  }

  public function get_comment_date_query() {
    $time_from  = 0;
    $time_to    = 0;

    if( isset( $_POST[ 'wpep_post_comments_time_from'] ) )
      $time_from = intval( $_POST[ 'wpep_post_comments_time_from'] );
    else if( isset( $_GET[ 'wpep-post-comments-time-from' ] ) )
      $time_from = intval( $_GET[ 'wpep-post-comments-time-from' ] );

    if( isset( $_POST[ 'wpep_post_comments_time_to'] ) )
      $time_to = intval( $_POST[ 'wpep_post_comments_time_to'] );
    else if( isset( $_GET[ 'wpep-post-comments-time-to' ] ) )
      $time_to = intval( $_GET[ 'wpep-post-comments-time-to' ] );

    if( $time_from === 0 && $time_to === 0 )
      return null;

    $response = [
      'column'  => 'comment_date',
    ];

    if( $time_from !== 0 )
      $response[ 'after' ] = date( 'Y-m-d H:i:s', $time_from );

    if( $time_to !== 0 )
      $response[ 'before' ] = date( 'Y-m-d H:i:s', $time_to );

    return $response;
  }

}